const net = require('net');
const path = require('path');
const process = require('process');
const crypto = require('crypto');

const port = Number(process.argv[2]);
const key = process.argv[3];

function handle() {
    const toJSON = (obj) => {
        payload = JSON.stringify(obj);
        const hmac = crypto.createHmac('sha256', key);
        hmac.update(payload);
        const hmac_b64 = hmac.digest('base64');
        return JSON.stringify({hmac: hmac_b64, payload}) + '\n';
    }
    const run = (file) => toJSON({ type: 'run', content: file });
    const close = (file) => toJSON({ type: 'close', content: file || '' });
    const stop = () => toJSON({ type: 'stop', content: '' });
    const isopen = (file) => toJSON({ type: 'isopen', content: file });
    const isready = () => toJSON({ type: 'isready', content: '' });

    const notebook = (arg) => {
        if (arg) {
            return path.join(process.cwd(), arg);
        }
        throw new Error('No notebook specified.');
    }

    const type = process.argv[4];
    const arg = process.argv[5];

    switch (type) {
        case 'run':
            return run(notebook(arg));
        case 'close':
            return close(notebook(arg));
        case 'stop':
            return stop();
        case 'isopen':
            return isopen(notebook(arg));
        case 'isready':
            return isready();
        default:
            throw new Error('Invalid command.');
    }
}

const debug = false;

const client = new net.Socket();
client.connect(port, '127.0.0.1', function () {
    debug && console.log('Connected');
    client.write(handle());
});
client.on('data', function (data) {
    console.log(debug ? JSON.parse(data.toString()) : data.toString());
    client.destroy();
});
client.on('close', function () {
    debug && console.log('Connection closed');
});
client.on('error', function (err) {
    const type = process.argv[4];
    if (type == 'stop' && err.code == 'ECONNRESET'){
        debug && console.log('Connection was reset after sending the stop command.')
    } else {
        throw err;
    }
});
